typedef unsigned char BYTE;

#include <stdio.h>
#include <memory.h>

//  LZ
#define N_DATA		4096	//!<   
#define F_COUNT		60		//!<    
#define THRESHOLD	2		//!<   
#define NIL			N_DATA	//!<      (    )

//  ARI
#define M   16				//!<  
#define Q1  (1<<M-1)		//  
#define Q2  (2 * Q1)
#define Q3  (3 * Q1)
#define Q4  (4 * Q1)

#define MAX_CUM (Q1-1)		//!<    
#define N_CHAR  (256 - THRESHOLD + F_COUNT) //!<   


class LZARI{
BYTE text_buf[N_DATA+F_COUNT-1];	//!<    
int match_position,match_length;	//   (   InsertNode)
int lson[N_DATA+1],rson[N_DATA+257],par[N_DATA+1]; //   (    256  rson)
// ari
unsigned int low,high,value;	//   
int shifts;  //   
int char_to_sym[N_CHAR],sym_to_char[N_CHAR+1]; //    (   )
unsigned int sym_freq[N_CHAR+1];  //!<  
unsigned int sym_cum[N_CHAR+1];   //!<   
unsigned int position_cum[N_DATA+1]; //!<   
// vars
	int textsize,codesize,printcount;		
	unsigned int byte,mask;	//   
// file
	FILE *infile,*outfile; 
	void PutBit(int bit);				//!<  
	int  GetBit();						//!<  
	void StartRead();					//!<   
	void StartWrite();					//!<   
	void StopWrite();					//!<   
// ARI
	void StartModel();					//!<   . 
// LZ+tree
	void InitTree();					//!<  
// func
	void InsertNode(int r);				//!<    
	void DeleteNode(int p);				//!<    
	void UpdateModel(int sym);			//!<   . 
	void Output(int bit);				//!<    
	void EncodeChar(int ch);			//!<   ( )  . 
	void EncodePosition(int position);	//!<    . 
	void EncodeEnd();					//!<   . 
	int  SearchSym(unsigned int x);		//!<     ( . )
	int  SearchPos(unsigned int x);		//!<     ( . )
	void StartDecode();					//!<  . 
	int  DecodeChar();					//!<   ( )  . 
	int  DecodePosition();				//!<    . 
// misk
	void Init();						//!<   
public:
	bool Encode(FILE *infile,FILE *outfile);	//!< 
	bool Decode(FILE *infile,FILE *outfile);	//!< 
};


void LZARI::Init()
{
	textsize=codesize=printcount=0;
//	low=0; high=Q4; value=0; shifts=0;
}

// IO filesys ================================

void LZARI::PutBit(int bit) 
{
	if(bit) byte|=mask;
	if (!(mask>>=1)) putc(byte,outfile),byte=0,mask=0x80,codesize++;
}

int LZARI::GetBit()  
{
    if(!(mask >>= 1)) byte=getc(infile),mask=0x80;
    return (byte&mask)!=0;
}

void LZARI::StartRead()
{
	mask=0;
}

void LZARI::StartWrite()
{
	mask=0x80,byte=0;
}

void LZARI::StopWrite()  
{
	for(int n=0; n<7; n++) PutBit(0);
}


//********** LZSS with multiple binary trees **********

void LZARI::InitTree()  
{
	int i;
	for(i=N_DATA+1; i<=N_DATA+256; i++) rson[i]=NIL;       // root
	for(i=0; i<N_DATA; i++) par[i]=NIL;   // node 
}

//    .
//         .
void LZARI::InsertNode(int r)
{
	int  i,p,cmp,temp; BYTE  *key;
	cmp=1;  key=&text_buf[r]; p=N_DATA+1+key[0];
	rson[r]=lson[r]=NIL; match_length=0;
	for(;;) {
		if(cmp>=0) {
			if(rson[p]!=NIL) p=rson[p]; else {rson[p]=r;par[r]=p;return;}
		} else {
			if(lson[p]!=NIL) p=lson[p]; else {lson[p]=r;par[r]=p;return;}
		}
		for(i=1; i<F_COUNT; i++) if ((cmp=key[i]-text_buf[p+i])) break;
		if (i>THRESHOLD) {
			if(i>match_length) {
				match_position=(unsigned int)(r-p)%N_DATA;
				match_length=i;
				if(match_length>=F_COUNT) break;
			} else if (i==match_length) {
				temp=(unsigned int)(r-p)%N_DATA;
				if(temp<match_position) match_position=temp;
			}
	}}
	par[r]=par[p]; lson[r]=lson[p]; rson[r]=rson[p];
	par[lson[p]]=par[rson[p]]=r;
	if(rson[par[p]]==p) rson[par[p]]=r;
	else lson[par[p]]=r;
	par[p]=NIL;  // remove p 
}


void LZARI::DeleteNode(int p)  
{
	int q;
	if(par[p]==NIL) return;  
	if(rson[p]==NIL) q=lson[p];
	else if(lson[p]==NIL) q=rson[p];
	else {
		q=lson[p];
		if(rson[q]!=NIL) {
			do{q=rson[q];} while(rson[q]!=NIL);
			rson[par[q]]=lson[q]; par[lson[q]]=par[q];
			lson[q]=lson[p]; par[lson[p]]=q;
		}
		rson[q]=rson[p]; par[rson[p]]=q;
	}
	par[q]=par[p];
	if(rson[par[p]]==p) rson[par[p]]=q; else lson[par[p]]=q;
	par[p]=NIL;
}

//********** Arithmetic Compression **********
//  If you are not familiar with arithmetic compression, you should read
//                I. E. Witten, R. M. Neal, and J. G. Cleary,
//                       Communications of the ACM, Vol. 30, pp. 520-540 (1987),
//       from which much have been borrowed.  

void LZARI::StartModel()  
{
	int ch, sym, i;
//  
	sym_cum[N_CHAR]=0;
	for(sym=N_CHAR; sym>=1; sym--) {
		ch=sym-1;
		char_to_sym[ch]=sym; sym_to_char[sym]=ch;
		sym_freq[sym]=1;
		sym_cum[sym-1]=sym_cum[sym]+sym_freq[sym];
	}
	sym_freq[0]=0;  /* sentinel (!= sym_freq[1]) */
	position_cum[N_DATA]=0;
	for(i=N_DATA; i>=1; i--) position_cum[i-1]=position_cum[i]+10000/(i+200);
//   
	low=0,high=Q4,value=0;shifts=0;
}

void LZARI::UpdateModel(int sym)
{
	int i,c,ch_i,ch_sym;
	if(sym_cum[0]>=MAX_CUM) {
		c=0;
		for(i=N_CHAR; i>0; i--) sym_cum[i]=c,c+=(sym_freq[i]=(sym_freq[i]+1)>>1);
		sym_cum[0]=c;
	}
	for(i=sym; sym_freq[i]==sym_freq[i-1]; i--) ;
	if(i<sym) {
		ch_i=sym_to_char[i];    ch_sym=sym_to_char[sym];
		sym_to_char[i]=ch_sym;  sym_to_char[sym]=ch_i;
		char_to_sym[ch_i]=sym;  char_to_sym[ch_sym]=i;
	}
	sym_freq[i]++;
	while(--i>=0) sym_cum[i]++;
}


void LZARI::Output(int bit)  // Output 1 bit, followed by its complements
{
	PutBit(bit);
	for(;shifts>0; shifts--) PutBit(!bit);
}


void LZARI::EncodeChar(int ch)
{
	int sym,range;
	sym=char_to_sym[ch];
	range=high-low;
	high=low+(range*sym_cum[sym-1])/sym_cum[0];
	low+=    (range*sym_cum[sym  ])/sym_cum[0];
	for(;;) {
		if(high<=Q2) Output(0);
		else if(low>=Q2) Output(1),low-=Q2,high-=Q2;
		else if(low>=Q1 && high<=Q3) shifts++,low-=Q1,high-=Q1;
		else break;
		low+=low,high+=high;
	}
	UpdateModel(sym);
}


void LZARI::EncodePosition(int position)
{
	int range;
	range=high-low;
	high=low+(range*position_cum[position  ])/position_cum[0];
	low+=    (range*position_cum[position+1])/position_cum[0];
	for(;;) {
		if(high<=Q2) Output(0);
		else if(low>=Q2) Output(1),low-=Q2,high-=Q2;
		else if(low>=Q1 && high<=Q3) shifts++,low-=Q1,high-=Q1;
		else break;
		low+=low,high+=high;
	}
}


void LZARI::EncodeEnd()
{
	shifts++;
	if(low<Q1) Output(0); else Output(1);
	StopWrite();  // flush bits remaining in buffer 
}


int LZARI::SearchSym(unsigned int x)
{
	int i,j,k;
	i=1; j=N_CHAR;
	while(i<j) {
		k=(i+j)/2;
		if(sym_cum[k]>x) i=k+1; else j=k;
	}
	return i;
}


int LZARI::SearchPos(unsigned int x)
{
	int i,j,k;
	i=1; j=N_DATA;
	while(i<j) {
		k=(i+j)/2;
		if(position_cum[k]>x) i=k+1; else j=k;
	}
	return i-1;
}

void LZARI::StartDecode()
{
	for(int i=0; i<M+1; i++) value=2*value+GetBit();
}


int LZARI::DecodeChar()
{
	int sym,ch,range;
	range=high-low;
	sym =SearchSym(((value-low+1)*sym_cum[0]-1)/range);
	high=low+(range*sym_cum[sym-1])/sym_cum[0];
	low+=    (range*sym_cum[sym  ])/sym_cum[0];
	for(;;) {
		if(low>=Q2) value-=Q2,low-=Q2,high-=Q2;
		else if(low>=Q1 && high<=Q3) value-=Q1,low-=Q1,high-=Q1;
		else if(high>Q2) break;
		low+=low,high+=high;
		value=2*value+GetBit();
	}
	ch=sym_to_char[sym];
	UpdateModel(sym);
	return ch;
}

int LZARI::DecodePosition()
{
	int position,range;
	range=high-low;
	position=SearchPos(((value-low+1)*position_cum[0]-1)/range);
	high=low+(range*position_cum[position  ])/position_cum[0];
	low+=    (range*position_cum[position+1])/position_cum[0];
	for(;;) {
		if(low>=Q2) value-=Q2,low-=Q2,high-=Q2;
		else if(low>=Q1 && high<=Q3) value-=Q1,low-=Q1,high-=Q1;
		else if(high>Q2) break;
		low+=low,high+=high;
		value=2*value+GetBit();
	}
	return position;
}



//********** Encode and Decode **********


bool LZARI::Encode(FILE *in,FILE *out)
{
	unsigned int r,s;
	int i,c,len,last_match_length;
	infile=in; outfile=out;
	Init();
	fseek(infile, 0L, SEEK_END); textsize=ftell(infile);
	if(fwrite(&textsize,1,sizeof(textsize),outfile)<1) return true;
	if(!textsize) return true;
	codesize+=sizeof(textsize);
	rewind(infile); 
	StartWrite(); StartModel(); InitTree();
	s=0; r=N_DATA-F_COUNT; memset(text_buf,' ',r);
	for(len=0; len<F_COUNT && (c=getc(infile))!=EOF; len++) text_buf[r+len]=c;
	textsize=len;
	for(i=1; i<=F_COUNT; i++) InsertNode(r-i);
	InsertNode(r);
	do {
		if(match_length>len) match_length=len;
		if(match_length<=THRESHOLD) match_length=1,EncodeChar(text_buf[r]);
		else EncodeChar(255-THRESHOLD+match_length),EncodePosition(match_position-1);
		last_match_length=match_length;
		for(i=0; i<last_match_length && (c=getc(infile))!=EOF; i++) { 
			DeleteNode(s); text_buf[s]=c;
			if(s<F_COUNT-1) text_buf[s+N_DATA]=c;
			s=(s+1)%N_DATA;
			r=(r+1)%N_DATA;
			InsertNode(r);
		}
		if((textsize+=i)>printcount) printf("%u\r",(int)textsize),printcount+=1024;
		while(i++<last_match_length) {
			DeleteNode(s);
			s=(s+1)%N_DATA;
			r=(r+1)%N_DATA;
			if(--len) InsertNode(r);
		}
	}while(len>0);
	EncodeEnd();
	printf("In : %u bytes\n",textsize);
	printf("Out: %u bytes\n",codesize);
	printf("Out/In: %.3f\n",(double)codesize/textsize);
	return false;
}


bool LZARI::Decode(FILE *in,FILE *out)
{
	infile=in; outfile=out;
	Init();
	unsigned int r;
	int i,j,k,c,count=0;
	if(fread(&textsize,1,sizeof(textsize),infile)<1) return true;  
	if(!textsize) return true;
	StartRead(); StartModel(); StartDecode();  
	r=N_DATA-F_COUNT; memset(text_buf,' ',r);
	for(count=0; count<textsize;) {
		c=DecodeChar();
		if(c<256){
			putc(c,outfile); text_buf[r++]=c;
			r%=N_DATA; count++;
		}else{
			i=(r-DecodePosition()-1)%N_DATA;
			j=c-255+THRESHOLD;
			for(k=0; k<j; k++) {
				c=text_buf[(i+k)%N_DATA];
				putc(c,outfile); text_buf[r++]=c;
				r%=N_DATA; count++;
			}
		}
		if(count>printcount) printf("%12lu\r", count),printcount+=1024;
	}
	printf("%12lu\n",count);
	return false;
}





void main()
{

	LZARI pack;
	FILE *in,*out;

	in=fopen("1.xml","rb");
	out=fopen("1.bin","wb");
	pack.Encode(in,out);
	fclose(out);
	fclose(in);

	in=fopen("1.bin","rb");
	out=fopen("1.res","wb");
	pack.Decode(in,out);
	fclose(out);
	fclose(in);
/*
	in=fopen("excel.exe","rb");
	out=fopen("2.bin","wb");
	pack.Encode(in,out);
	fclose(out);
	fclose(in);

	in=fopen("2.bin","rb");
	out=fopen("2.res","wb");
	pack.Decode(in,out);
	fclose(out);
	fclose(in);
	*/
}
